package cviceni3;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 *
 * @author adam
 */
public class Cviceni3 {

    /**
     * W3C object model representation of a xml document.
     */
    private Document doc;

    /*
     * prida korenovy element <zvirata>
     * @return korenovy element
     */
    public Element pridejKoren() {
        Element root = doc.createElement("zvirata");
        doc.appendChild(root);
        return root;
    }

    /*
     * prida element <pes> a nastavi mu vlastnost druh
     * @param root korenovy element
     * @param druh nazev druhu
     * @return element psa
     */
    public Element pridejPsa(Element root, String druh) {
        Element pes = doc.createElement("pes");
        pes.setAttribute("druh", druh);
        root.appendChild(pes);
        return pes;
    }

    /*
     * prida element <barva> a nastavi textovou hodnotu
     * @param pes rodicovsky element
     * @param barva textova hodnota barvy
     * @return element barvy
     */
    public Element pridejBarvu(Element pes, String barva) {
        Element barvaElement = doc.createElement("barva");
        Text barvaText = doc.createTextNode(barva);
        barvaElement.appendChild(barvaText);
        pes.appendChild(barvaElement);
        return barvaElement;
    }

    /**
     * Constructor tridy, vytvori prazdny DOM dokument
     */
    private Cviceni3() throws SAXException, ParserConfigurationException {
        // Vytvorime instanci tovarni tridy
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        // Pomoci tovarni tridy ziskame instanci DocumentBuilderu
        DocumentBuilder builder = factory.newDocumentBuilder();
        // DocumentBuilder pouzijeme pro zpracovani XML dokumentu
        // a ziskame model dokumentu ve formatu W3C DOM
        doc = builder.newDocument();
    }

    /**
     * Constructor tridy, vytvori prazdny DOM dokument
     */
    public void validateParse(String fileName) throws SAXException, ParserConfigurationException, IOException {
        // Vytvorime instanci tovarni tridy
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setValidating(true);
        // Pomoci tovarni tridy ziskame instanci DocumentBuilderu
        DocumentBuilder builder = factory.newDocumentBuilder();
        // DocumentBuilder pouzijeme pro zpracovani XML dokumentu
        // a ziskame model dokumentu ve formatu W3C DOM
        builder.setErrorHandler(new org.xml.sax.ErrorHandler() {
            public void fatalError(SAXParseException exception) throws SAXException {
                System.out.println("Line: " + exception.getLineNumber() + "\nFatal Error: " + exception.getMessage());
            }
            public void error(SAXParseException e) throws SAXParseException {
                System.out.println("Line: " + e.getLineNumber() + "\nError: " + e.getMessage());
            }
            public void warning(SAXParseException err) throws SAXParseException {
                System.out.println("Line: " + err.getLineNumber() + "\nWarning: " + err.getMessage());
            }
        });
        builder.parse(fileName);
    }


    /*
     * Z dokumentu ve formatu DOM vytvori XML dokument a zapise ho do souboru
     * s urcenym URI
     */
    public void serializetoXML(String fileName) throws IOException,
            TransformerConfigurationException, TransformerException {
        // Vytvorime instanci tovarni tridy
        TransformerFactory factory = TransformerFactory.newInstance();
        // Pomoci tovarni tridy ziskame instanci tzv. kopirovaciho transformeru
        Transformer transformer = factory.newTransformer();
        // Vstupem transformace bude dokument v pameti
        DOMSource source = new DOMSource(doc);
        // Vystupem transformace bude vystupni soubor
        StreamResult result = new StreamResult(new FileOutputStream(fileName));
        transformer.setOutputProperty(OutputKeys.INDENT, "yes");
        transformer.setOutputProperty(OutputKeys.DOCTYPE_SYSTEM, "zvirata.dtd");

        // Provedeme transformaci
        transformer.transform(source, result);
    }

    /*
     * Z dokumentu ve formatu DOM vytvori XML dokument a zapise ho do souboru
     */
    public void serializetoXML(File output) throws IOException,
            TransformerException {
        serializetoXML(output.toString());
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) throws IOException, SAXException,
            ParserConfigurationException, TransformerException {

        Cviceni3 ukazka = new Cviceni3();

        Element koren = ukazka.pridejKoren();

        Element pes1 = ukazka.pridejPsa(koren, "pudl");
        Element pes2 = ukazka.pridejPsa(koren, "bernardyn");
        Element pes3 = ukazka.pridejPsa(koren, "jezevcik");
        ukazka.pridejBarvu(pes1, "cerna");
        ukazka.pridejBarvu(pes2, "hnedobila");
        ukazka.pridejBarvu(pes3, "hneda");

        ukazka.serializetoXML("output.xml");

        ukazka.pridejPsa(pes3, "dalmatin");

        ukazka.serializetoXML("output2.xml");

        ukazka.validateParse("output.xml");
        ukazka.validateParse("output2.xml");
    }
}
